import { InjectionToken, EventEmitter, Component, ViewEncapsulation, ChangeDetectionStrategy, ChangeDetectorRef, ElementRef, Renderer2, Inject, PLATFORM_ID, Optional, Input, Output, Directive, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { trigger, transition } from '@angular/animations';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { isPlatformBrowser, CommonModule } from '@angular/common';
import { fromOutsideTouch, fromOutsideClick, slideInLeft, slideInRight, slideInUp, slideInDown, slideOutLeft, slideOutRight, slideOutUp, slideOutDown } from '@ng-tangram/components/core';

const NT_DRAWER_CONTAINER = new InjectionToken('nt-drawer-container');
const ANIMATION_TIMING = 0.4;
let uniqueId = 0;
class NtDrawerComponent {
    constructor(_changeDetectorRef, _element, _renderer, _platformId, container) {
        this._changeDetectorRef = _changeDetectorRef;
        this._element = _element;
        this._renderer = _renderer;
        this._platformId = _platformId;
        this.id = `nt-drawer-${uniqueId++}`;
        this._destory = new Subject();
        this._backdrop = false;
        this._touchmode = false;
        this.state = 'closed';
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        if (isPlatformBrowser(_platformId)) {
            this._initContainerAndStyles(container);
        }
        this._changePlacementAndStyles('left');
    }
    get backdrop() { return this._backdrop; }
    set backdrop(value) {
        this._backdrop = coerceBooleanProperty(value);
        if (this._backdrop) {
            this._createBackdropOverlay();
        }
        else {
            this._removeBackdropOverlay();
        }
    }
    get placement() { return this._placement; }
    set placement(value) {
        this._changePlacementAndStyles(value);
    }
    get touchmode() { return this._touchmode; }
    set touchmode(value) {
        this._touchmode = coerceBooleanProperty(value);
    }
    ngAfterViewInit() { }
    ngOnDestroy() {
        if (isPlatformBrowser(this._platformId)) {
            this._renderer.removeClass(this._container, `${this.id}-container`);
        }
        this._destory.next();
        this._destory.complete();
        this._unsubscribeOutsideActionEvent();
    }
    open() {
        this.state = this.placement;
    }
    close() {
        this.state = 'closed';
    }
    onAnimationDone(event) {
        if (event.fromState === 'void') {
            return;
        }
        if (event.toState !== 'closed') {
            this._subscribeOutsideActionEvent();
            this.afterOpen.emit();
        }
        else {
            this.afterClosed.emit();
        }
    }
    onAnimationStart(event) {
        if (event.fromState === 'void') {
            return;
        }
        if (event.toState === 'closed') {
            this._unsubscribeOutsideActionEvent();
            this._disattachBackdropOverlay();
            this.beforeClosed.emit();
        }
        else {
            this._attachBackdropOverlay();
            this.beforeOpen.emit();
        }
    }
    _changePlacementAndStyles(placement) {
        if (this._placement) {
            this._renderer.removeClass(this._element.nativeElement, this._placement);
        }
        this._renderer.addClass(this._element.nativeElement, this._placement = placement);
    }
    _initContainerAndStyles(container) {
        this._container = container ? container.element : document.body;
        if (this._container !== document.body) {
            this._renderer.addClass(this._element.nativeElement, 'nested');
        }
        this._renderer.addClass(this._container, `${this.id}-container`);
    }
    _createBackdropOverlay() {
        this._backdropElement = this._renderer.createElement('div');
        this._renderer.addClass(this._backdropElement, 'nt-drawer-backdrop');
        this._renderer.addClass(this._backdropElement, `${this.id}-backdrop`);
    }
    _removeBackdropOverlay() {
        this._disattachBackdropOverlay();
        this._backdropElement = null;
    }
    _attachBackdropOverlay() {
        if (this.backdrop && this._backdropElement) {
            this._renderer.appendChild(this._container, this._backdropElement);
            this._renderer.addClass(this._container, 'nt-drawer-scrollblock');
        }
    }
    _disattachBackdropOverlay() {
        if (this._backdropElement) {
            this._renderer.removeChild(this._container, this._backdropElement);
            this._renderer.removeClass(this._container, 'nt-drawer-scrollblock');
        }
    }
    _subscribeOutsideActionEvent() {
        const outsideAction = this.touchmode ? fromOutsideTouch : fromOutsideClick;
        this._outsideActionSubscription = outsideAction([this._element.nativeElement], this._container)
            .pipe(takeUntil(this._destory))
            .subscribe(_ => this.close());
    }
    _unsubscribeOutsideActionEvent() {
        if (this._outsideActionSubscription) {
            this._outsideActionSubscription.unsubscribe();
            this._outsideActionSubscription = null;
        }
    }
}
NtDrawerComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-drawer',
                template: `<ng-content></ng-content>`,
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                animations: [
                    trigger('slide', [
                        transition('closed => left', slideInLeft(ANIMATION_TIMING)),
                        transition('closed => right', slideInRight(ANIMATION_TIMING)),
                        transition('closed => top', slideInUp(ANIMATION_TIMING)),
                        transition('closed => bottom', slideInDown(ANIMATION_TIMING)),
                        transition('left => closed', slideOutLeft(ANIMATION_TIMING)),
                        transition('right => closed', slideOutRight(ANIMATION_TIMING)),
                        transition('top => closed', slideOutUp(ANIMATION_TIMING)),
                        transition('bottom => closed', slideOutDown(ANIMATION_TIMING))
                    ])
                ],
                host: {
                    'class': 'nt-drawer',
                    '[class.opened]': 'state !== "closed"',
                    '[class.backdrop]': 'backdrop',
                    '[@slide]': 'state',
                    '(@slide.start)': 'onAnimationStart($event)',
                    '(@slide.done)': 'onAnimationDone($event)',
                }
            },] }
];
NtDrawerComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: ElementRef },
    { type: Renderer2 },
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DRAWER_CONTAINER,] }] }
];
NtDrawerComponent.propDecorators = {
    backdrop: [{ type: Input }],
    placement: [{ type: Input }],
    touchmode: [{ type: Input }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }]
};

class NtDrawerContainerDirective {
    constructor(_element) {
        this.element = _element.nativeElement;
    }
}
NtDrawerContainerDirective.decorators = [
    { type: Directive, args: [{
                selector: '[nt-drawer-container]',
                providers: [
                    { provide: NT_DRAWER_CONTAINER, useExisting: NtDrawerContainerDirective }
                ],
                host: {
                    'class': 'nt-drawer-container'
                }
            },] }
];
NtDrawerContainerDirective.ctorParameters = () => [
    { type: ElementRef }
];

class NtDrawerModule {
}
NtDrawerModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                exports: [
                    NtDrawerComponent,
                    NtDrawerContainerDirective
                ],
                declarations: [
                    NtDrawerComponent,
                    NtDrawerContainerDirective
                ]
            },] }
];

export { NT_DRAWER_CONTAINER, NtDrawerComponent, NtDrawerContainerDirective, NtDrawerModule };
//# sourceMappingURL=drawer.js.map
