import { BehaviorSubject, merge } from 'rxjs';
import { map, take } from 'rxjs/operators';
import { DataSource } from '@angular/cdk/collections';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { CdkTreeNode, CdkTree, CdkTreeNodeDef, CdkNestedTreeNode, CdkTreeNodeOutlet, CDK_TREE_NODE_OUTLET_NODE, CdkTreeNodePadding, CdkTreeNodeToggle, CdkTreeModule } from '@angular/cdk/tree';
import { Directive, ElementRef, Attribute, Input, IterableDiffers, ViewContainerRef, Inject, Optional, Component, ViewEncapsulation, ChangeDetectionStrategy, ViewChild, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

class NtTreeNestedDataSource extends DataSource {
    constructor() {
        super(...arguments);
        this._data = new BehaviorSubject([]);
    }
    get data() { return this._data.value; }
    set data(value) { this._data.next(value); }
    connect(collectionViewer) {
        return merge(...[collectionViewer.viewChange, this._data])
            .pipe(map(() => this.data));
    }
    disconnect() {
    }
}

class NtTreeFlattener {
    constructor(transformFunction, getLevel, isExpandable, getChildren) {
        this.transformFunction = transformFunction;
        this.getLevel = getLevel;
        this.isExpandable = isExpandable;
        this.getChildren = getChildren;
    }
    _flattenNode(node, level, resultNodes, parentMap) {
        const flatNode = this.transformFunction(node, level);
        resultNodes.push(flatNode);
        if (this.isExpandable(flatNode)) {
            const childrenNodes = this.getChildren(node);
            if (Array.isArray(childrenNodes)) {
                this._flattenChildren(childrenNodes, level, resultNodes, parentMap);
            }
            else {
                childrenNodes.pipe(take(1)).subscribe(children => {
                    this._flattenChildren(children, level, resultNodes, parentMap);
                });
            }
        }
        return resultNodes;
    }
    _flattenChildren(children, level, resultNodes, parentMap) {
        children.forEach((child, index) => {
            let childParentMap = parentMap.slice();
            childParentMap.push(index !== children.length - 1);
            this._flattenNode(child, level + 1, resultNodes, childParentMap);
        });
    }
    flattenNodes(structuredData) {
        let resultNodes = [];
        structuredData.forEach(node => this._flattenNode(node, 0, resultNodes, []));
        return resultNodes;
    }
    expandFlattenedNodes(nodes, treeControl) {
        let results = [];
        let currentExpand = [];
        currentExpand[0] = true;
        nodes.forEach(node => {
            let expand = true;
            for (let i = 0; i <= this.getLevel(node); i++) {
                expand = expand && currentExpand[i];
            }
            if (expand) {
                results.push(node);
            }
            if (this.isExpandable(node)) {
                currentExpand[this.getLevel(node) + 1] = treeControl.isExpanded(node);
            }
        });
        return results;
    }
}
class NtTreeFlatDataSource extends DataSource {
    constructor(treeControl, treeFlattener, initialData = []) {
        super();
        this.treeControl = treeControl;
        this.treeFlattener = treeFlattener;
        this._flattenedData = new BehaviorSubject([]);
        this._expandedData = new BehaviorSubject([]);
        this._data = new BehaviorSubject(initialData);
    }
    get data() { return this._data.value; }
    set data(value) {
        this._data.next(value);
        this._flattenedData.next(this.treeFlattener.flattenNodes(this.data));
        this.treeControl.dataNodes = this._flattenedData.value;
    }
    connect(collectionViewer) {
        const changes = [
            collectionViewer.viewChange,
            this.treeControl.expansionModel.changed,
            this._flattenedData
        ];
        return merge(...changes).pipe(map(() => {
            this._expandedData.next(this.treeFlattener.expandFlattenedNodes(this._flattenedData.value, this.treeControl));
            return this._expandedData.value;
        }));
    }
    disconnect() { }
}

class NtTreeNodeDirective extends CdkTreeNode {
    constructor(_elementRef, _tree, tabIndex) {
        super(_elementRef, _tree);
        this._elementRef = _elementRef;
        this._tree = _tree;
        this._disabled = false;
        this._tabIndex = 0;
        this.role = 'treeitem';
        this.tabIndex = Number(tabIndex) || 0;
    }
    get disabled() { return this._disabled; }
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    get tabIndex() { return this.disabled ? -1 : this._tabIndex; }
    set tabIndex(value) { this._tabIndex = value != null ? value : 0; }
}
NtTreeNodeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-tree-node, [nt-tree-node]',
                exportAs: 'ntTreeNode',
                inputs: ['disabled', 'tabIndex'],
                host: {
                    'class': 'nt-tree-node'
                },
                providers: [{ provide: CdkTreeNode, useExisting: NtTreeNodeDirective }]
            },] }
];
NtTreeNodeDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: CdkTree },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] }
];
NtTreeNodeDirective.propDecorators = {
    role: [{ type: Input }]
};
class NtTreeNodeDefDirective extends CdkTreeNodeDef {
}
NtTreeNodeDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntTreeNodeDef]',
                inputs: [
                    'when: ntTreeNodeDefWhen'
                ],
                providers: [{ provide: CdkTreeNodeDef, useExisting: NtTreeNodeDefDirective }]
            },] }
];
NtTreeNodeDefDirective.propDecorators = {
    data: [{ type: Input, args: ['ntTreeNode',] }]
};
class NtNestedTreeNodeDirective extends CdkNestedTreeNode {
    constructor(_elementRef, _tree, _differs, tabIndex) {
        super(_elementRef, _tree, _differs);
        this._elementRef = _elementRef;
        this._tree = _tree;
        this._differs = _differs;
        this._disabled = false;
        this._tabIndex = 0;
        this.tabIndex = Number(tabIndex) || 0;
    }
    get disabled() { return this._disabled; }
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    get tabIndex() { return this.disabled ? -1 : this._tabIndex; }
    set tabIndex(value) { this._tabIndex = value != null ? value : 0; }
    ngAfterContentInit() {
        super.ngAfterContentInit();
    }
    ngOnDestroy() {
        super.ngOnDestroy();
    }
}
NtNestedTreeNodeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-nested-tree-node, [nt-nested-tree-node]',
                exportAs: 'ntNestedTreeNode',
                host: {
                    'class': 'nt-nested-tree-node',
                },
                inputs: ['disabled', 'tabIndex'],
                providers: [
                    { provide: CdkNestedTreeNode, useExisting: NtNestedTreeNodeDirective },
                    { provide: CdkTreeNode, useExisting: NtNestedTreeNodeDirective }
                ]
            },] }
];
NtNestedTreeNodeDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: CdkTree },
    { type: IterableDiffers },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] }
];
NtNestedTreeNodeDirective.propDecorators = {
    node: [{ type: Input, args: ['ntNestedTreeNode',] }]
};

class NtTreeNodeOutletDirective {
    constructor(viewContainer, _node) {
        this.viewContainer = viewContainer;
        this._node = _node;
    }
}
NtTreeNodeOutletDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntTreeNodeOutlet]',
                providers: [
                    { provide: CdkTreeNodeOutlet, useExisting: NtTreeNodeOutletDirective }
                ]
            },] }
];
NtTreeNodeOutletDirective.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: undefined, decorators: [{ type: Inject, args: [CDK_TREE_NODE_OUTLET_NODE,] }, { type: Optional }] }
];

class NtTreeNodePaddingDirective extends CdkTreeNodePadding {
}
NtTreeNodePaddingDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntTreeNodePadding]',
                providers: [{ provide: CdkTreeNodePadding, useExisting: NtTreeNodePaddingDirective }]
            },] }
];
NtTreeNodePaddingDirective.propDecorators = {
    level: [{ type: Input, args: ['ntTreeNodePadding',] }],
    indent: [{ type: Input, args: ['ntTreeNodePaddingIndent',] }]
};

class NtTreeNodeToggleDirective extends CdkTreeNodeToggle {
    constructor() {
        super(...arguments);
        this.recursive = false;
    }
}
NtTreeNodeToggleDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntTreeNodeToggle]',
                providers: [{ provide: CdkTreeNodeToggle, useExisting: NtTreeNodeToggleDirective }]
            },] }
];
NtTreeNodeToggleDirective.propDecorators = {
    recursive: [{ type: Input, args: ['ntTreeNodeToggleRecursive',] }]
};

class NtTreeComponent extends CdkTree {
}
NtTreeComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-tree, [nt-tree]',
                template: `<ng-container ntTreeNodeOutlet></ng-container>`,
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [
                    { provide: CdkTree, useExisting: NtTreeComponent }
                ],
                host: {
                    'class': 'nt-tree'
                }
            },] }
];
NtTreeComponent.propDecorators = {
    _nodeOutlet: [{ type: ViewChild, args: [NtTreeNodeOutletDirective, { static: true },] }]
};

const NT_TREE_DIRECTIVES = [
    NtNestedTreeNodeDirective,
    NtTreeNodeDefDirective,
    NtTreeNodePaddingDirective,
    NtTreeNodeToggleDirective,
    NtTreeComponent,
    NtTreeNodeDirective,
    NtTreeNodeOutletDirective
];
class NtTreeModule {
}
NtTreeModule.decorators = [
    { type: NgModule, args: [{
                imports: [CdkTreeModule, CommonModule],
                exports: NT_TREE_DIRECTIVES,
                declarations: NT_TREE_DIRECTIVES
            },] }
];

export { NtNestedTreeNodeDirective, NtTreeComponent, NtTreeFlatDataSource, NtTreeFlattener, NtTreeModule, NtTreeNestedDataSource, NtTreeNodeDefDirective, NtTreeNodeDirective, NtTreeNodeOutletDirective, NtTreeNodePaddingDirective, NtTreeNodeToggleDirective };
//# sourceMappingURL=tree.js.map
